#![cfg_attr(nightly, feature(never_type))] 

#![allow(dead_code)]

use cfg_if::cfg_if;
use lazy_static::lazy_static;
//use futures::prelude::*;
use tokio::prelude::*;
use futures::{
    Future,
};

#[allow(unused_imports)]
use color_eyre::{
    eyre::{
	self,
	eyre,
	WrapErr as _,
    },
    SectionExt as _, Help as _,
};
use std::{
    sync::Arc,
};

mod ext;
use ext::*;
mod util;
mod dedup;

mod resolve;
mod database;

mod args;
mod config;


cfg_if!{
    if #[cfg(nightly)] {
	type Never = !;
    } else {
	type Never = std::convert::Infallible;
    }
}

type Pointer = *const Never;

fn install() -> eyre::Result<()>
{
    color_eyre::install()?;

    Ok(())
}

async fn process(config: Arc<config::Config>, file: String)
{
    println!(" -> {:?}", file);
    let dbdir = resolve::mangle_path(&config, &file);
    println!("Database path for this file {:?}", dbdir);
    println!("Demangle: {:?}", resolve::demangle_path(&dbdir).await
	     .wrap_err(eyre!("Failed to demangle path"))
	     .with_section(|| dbdir.to_string_lossy().into_owned().header("Path was"))
	     .with_section(|| config.base_dir.clone().to_string_lossy().into_owned().header("The videl database root directory is"))
	     .with_suggestion(|| "Are you sure this database location is correct"));
}

async fn validate_config(config: config::Config) -> eyre::Result<config::Config>
{
    config.validate().await
	.with_section(|| format!("{:#?}", config).header("Config was"))
	.with_suggestion(|| "Are you sure the base directory is a valid pathspec?")
	.with_suggestion(|| "Are you sure the base directory is not already occupied by a non-directory?")
	.with_suggestion(|| format!("Are you sure we have read-write access to {:?}?", config.base_dir))?;
    Ok(config)
}

async fn begin() -> eyre::Result<i32>
{
    install()?;
    let config = Arc::new(validate_config(config::Config::default()).await
			  .wrap_err(eyre!("Failed to validate config"))?);
    println!("Validated config OK");
    if args::process(|file| {
	let config = Arc::clone(&config);
	process(config, file)
    }).await
        .wrap_err(eyre!("One or more child workers failed to complete successfully"))?
	.len() == 0
    {
	args::usage();
    }
    
    Ok(0)
}

#[tokio::main]
async fn main() -> eyre::Result<()>
{
    std::process::exit(match begin().await {
	Ok(0) => return Ok(()),
	Err(err) => {
	    eprintln!("\nexited with error: {}", err);
	    return Err(err)
		.wrap_err(eyre!("Fatal error"));
	},
	Ok(v) => v
    })
}
